package edu.nps.seed.nst
import TestEvent._
import scala.collection.mutable.Map
import scala.collection.mutable.ListBuffer
import cern.jet.random.Uniform



class Task(n: String, num:Int, d: Double) {
	val name = n
	val id = num
	val duration = d
	var startTime: Double = -1.
	var status = TaskStatus.Pending
	val successors = Map[String,Task]()
	val predecessors = Map[String, Task]()
	val resources = Map[String, Resource]()
 
	def start() = {
		println(TestEvent.currentTime+",check start task,"+this.name)
		if ( checkStartConditions() ) {
		  println(TestEvent.currentTime+",start task,"+this.name)
		  Task.scheduleEndTask(this)
		  status = TaskStatus.Active
		  startTime = TestEvent.currentTime
		  for (resource <- resources.values){
		    resource.startTask(this)
		  }
		}
  
  	}
 
	def checkStartConditions(): Boolean = {
	  // check if this task status is Active or Complete
	  if ( status == TaskStatus.Active || status == TaskStatus.Complete) {
	    println(TestEvent.currentTime+",task status,"+this.name+
	    			",task status "+this.status.toString)	    
	    return false
	  }
	  // check start time
	  if ( this.startTime > TestEvent.currentTime ) {
	    println(TestEvent.currentTime+",awaiting start time,"+this.name+
	    			",task start time "+this.startTime)
	    return false
	  }
		
	  //check predecessors complete
	  for (predecessor <- predecessors.values) {
	    if (predecessor.status != TaskStatus.Complete) {
	    	println(TestEvent.currentTime+",pending task,"+this.name+
                ",awaiting completion task "+predecessor.name)
	    	return false
	    }
	    	
	  }
	  //check resources available
	  for (resource <- resources.values) {
	    if ( resource.status == ResourceStatus.Busy) {
	      println(TestEvent.currentTime+",pending resource,"+this.name+
	      		",awaiting resource "+resource.name)
	      resource.addWaitingTask(this)
	      return false
	    }
	    
	  }
	  return true
	}
   
	def end() = {
		println(TestEvent.currentTime+",completed task,"+this.name)
	    for (successor <- successors.values if successor.name != "empty") {
	      Task.scheduleTaskNow(successor)
	    }
		for ( resource <- resources.values ) 
			resource.endTask(this)
	    status = TaskStatus.Complete
	}
   
 
	def addPredecessors(taskMap: Map[String, Task]) {
		for (predecessorName <- predecessors.keys) {
		  val predecessorTask = taskMap(predecessorName)
		  predecessors += (predecessorName -> predecessorTask)
		  predecessorTask.addSuccessor(taskMap(this.name))
		}
	  
	}
 
	def addResources(resourceMap: Map[String, Resource]) {
	  for (resourceName <- resources.keys) {
	    val resource = resourceMap(resourceName)
	    resources += (resourceName -> resource)
	  }
	  
	}
 
	def addPredecessor(task: Task) {
	    println("adding pre for task: "+this.name+" pre: "+task.name)
		predecessors += (task.name -> task)
	  
	}
 
	def addSuccessor(task: Task) {
	  println("adding suc for task: "+this.name+" suc: "+task.name)
	  successors += (task.name -> task)
	}
 
	def addPredecessor(taskName: String) {
	  predecessors += (taskName -> Task.emptyTask)
	}

	def addResource(resourceName: String){
	  resources += (resourceName -> Resource.emptyResource)
	}
   
	override def toString = {
	  val result = new StringBuilder
	  name addString result
	  "#id:" addString result
	  id.toString addString result
	  "#dur:" addString result
	  duration.toString addString result
	  resources.keys.toList addString (result, "#res:", "#res:", "")   
      predecessors.keys.toList addString (result, "#pre:","#pre:","")
   	  successors.keys.toList  addString (result, "#suc:", "#suc:", "")

	  result.toString
	}

    
}

object Task {
   val emptyTask = new Task("empty",0, 0.)
   var count: Int = 1
   
   def taskFromXML(node: scala.xml.Node): Task = {
	   val name = (node \ "name").text
	   val nominal_duration = (node \ "duration").text.toDouble
       val variation = TestEvent.duration_variability
       val range = nominal_duration*variation

       val duration = Uniform.staticNextDoubleFromTo(nominal_duration - range, nominal_duration + range)
	   val task = new Task(name, count, duration)
	   count += 1
    
	   val startTime = (node \ "startTime")
	   if ( startTime != Nil ) {
		   task.startTime = startTime.text.toDouble
	   }
       val predecessors = (node \ "predecessor")
       if ( predecessors != Nil) {
         for (predecessor <- predecessors ) {
           task.addPredecessor(predecessor.text)
         }
       }
       val resources = (node \ "resource")
       if ( resources != Nil) {
         for (resource <- resources ) {
           task.addResource(resource.text)
         }
       }

	   task
   } 
   
   def toEventList(): List[String] = {
     val taskEvents = new ListBuffer[String]()
      taskEvents += "id,name,startTime,endTime"
      
      val ts = TestEvent.taskMap.values.toList
      
	  for (task <- ts.sort((s,t) => s.id < t.id)) {
		  val endtime = task.startTime + task.duration
		  taskEvents += task.id+","+task.name+","+task.startTime.toString+","+endtime.toString
	  }     
     
     
     taskEvents.toList
   }
   
   
   def scheduleEndTask(task: Task) {
	   TestEvent.scheduleEndTask(task, TestEvent.currentTime + task.duration)
     
   }
   
   def scheduleStartTask(task: Task) {
     if ( task.startTime != -1.)
     	TestEvent.scheduleStartTask(task, task.startTime)
     
   }
   
   def scheduleTaskNow(task: Task) {
	   TestEvent.scheduleStartTaskNow(task)
   }
  
}

object TaskStatus extends Enumeration {
  val Complete = Value("Complete") 
  val Active = Value("Active")
  val AwaitingResource = Value("AwaitingResource")
  val Pending = Value("Pending") 
}

