package oldmcdata.agents;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.SimpleBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import oldmcdata.generators.Generator;
import oldmcdata.generators.GeneratorFactory;


/**
 * Title:        GeneratorAgent<p>
 * Description:  A simple agent that can send  custom messages to another agent.  Based on AgentSender in
 * JADE examples.receivers.AgentSender.<p>
 *Copyright: 2000 - 2005
 *         <p>
 * @author Stephen C. Upton
 * @version $Date: 12/4/2000  $ $Revision: 1.0 $
 *
 */

public class GeneratorAgent extends OldMcDataAgent {
    private Generator generator;

    protected void setup() {
        super.setup();
        this.addBehaviour(new InitialBehavior(this));
        System.out.println("this is a test of the GeneratorAgent arguments list: "
                + "StudyDir: " + studyDirName + " StudyFile: " + studyFileName);
    }


    class InitialBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public InitialBehavior(Agent a) {
            super(a);
        }

        public void action() {
            doInitialization();
            myAgent.addBehaviour(new DoGenerationBehavior(myAgent));
            finished = true;
        }

        private void doInitialization() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is in InitialBehavior");
            output.println("\nAgent " + myAgent.getLocalName() + "  is in InitialBehavior");
            generator = GeneratorFactory.getInstance(studyDirName, studyFileName);
            generator.setConfigFile(configFileName);
            generator.initialize();
        }

        public boolean done() {
            return finished;
        }
    }   // end InitialBehavior

    class RequestEvaluateBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public RequestEvaluateBehavior(Agent a) {
            super(a);
        }

        public void action() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is in RequestEvaluateBehavior");
            output.println("\nAgent " + myAgent.getLocalName() + "  is in RequestEvaluateBehavior");
            sendRequestEvaluateMessage();
            myAgent.addBehaviour(new WaitForEvaluationComplete(myAgent));
            finished = true;
        }

        private void sendRequestEvaluateMessage() {
            AID r = new AID();
            r.setLocalName("evaluator");
            ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
            msg.setSender(myAgent.getAID());
            msg.addReceiver(r);
            msg.setContent("DoExcursions");
            msg.setLanguage("PlainText");
            msg.setOntology("GEA");
            send(msg);
        }

        public boolean done() {
            return finished;
        }
    } // end RequestEvaluateBehavior

    class WaitForEvaluationComplete extends SimpleBehaviour {
        private boolean finished = false;
        private String message = "EvaluationComplete";

        public WaitForEvaluationComplete(Agent a) {
            super(a);
        }

        public void action() {
            output.println("\nAgent " + myAgent.getLocalName() + "  is in waiting for message: " + message);
            System.out.println("\nAgent " + myAgent.getLocalName() + " is in waiting for message: " + message);
//	try {
            MessageTemplate m1 = MessageTemplate.MatchOntology("GEA");

            //The agent waits for a specific message. If it doesn't arrive the behaviour is suspended until a new message arrives.
            ACLMessage msg = myAgent.receive(m1);

            if (msg != null) {
                System.out.println("\nAgent " + myAgent.getLocalName() + " is in waiting for message: " + message);
                output.println("\nAgent " + myAgent.getLocalName() + " is in waiting for message: " + message);
                output.println(msg.toString());
                System.out.println(msg.toString());
//        ContentElement content = myAgent.getContentManager().extractContent(msg);
                //       Object resp = l.get(0);
                //    List msgContent = myAgent.extractContent(msg);
                finished = true;
                myAgent.addBehaviour(new RequestAnalysisBehavior(myAgent));

            } else {
                output.println("\nAgent " + myAgent.getLocalName() + " has not received message: " + message);
                block();
            }
//    } // End of try
//    catch (FIPAException fe) {
//            System.err.println("FIPAException in fill/extract content:" + fe.getMessage());
//            System.exit(-1);
//    }
        }

        public boolean done() {
            return finished;
        }


    } // end WaitForEvaluationComplete

    class RequestAnalysisBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public RequestAnalysisBehavior(Agent a) {
            super(a);
        }

        public void action() {
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is in RequestAnalysisBehavior");
            output.println("\nAgent " + myAgent.getLocalName() + "  is in RequestAnalysisBehavior");
            sendRequestAnalysisMessage();
            myAgent.addBehaviour(new WaitForAnalysisComplete(myAgent));
            finished = true;
        }

        private void sendRequestAnalysisMessage() {
            AID r = new AID();
            r.setLocalName("analyzer");
            ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
            msg.setSender(myAgent.getAID());
            msg.addReceiver(r);
            msg.setContent("DoAnalysis");
            msg.setLanguage("PlainText");
            msg.setOntology("GEA");
            send(msg);
        }

        public boolean done() {
            return finished;
        }
    } // end RequestAnalysisBehavior

    class WaitForAnalysisComplete extends SimpleBehaviour {
        private boolean finished = false;
        private String message = "CompletedAnalysis";

        public WaitForAnalysisComplete(Agent a) {
            super(a);
        }

        public void action() {
            output.println("\nAgent " + myAgent.getLocalName() + "  is in waiting for message: " + message);
            System.out.println("\nAgent " + myAgent.getLocalName() + " is in waiting for message: " + message);
//	try {
            MessageTemplate m1 = MessageTemplate.MatchOntology("GEA");

            //The agent waits for a specific message. If it doesn't arrive the behaviour is suspended until a new message arrives.
            ACLMessage msg = myAgent.receive(m1);

            if (msg != null) {
                System.out.println("\nAgent " + myAgent.getLocalName() + " is in waiting for message: " + message);
                output.println("\nAgent " + myAgent.getLocalName() + " is in waiting for message: " + message);
                output.println(msg.toString());
                System.out.println(msg.toString());
//        ContentElement content = myAgent.getContentManager().extractContent(msg);
                //       Object resp = l.get(0);
                //    List msgContent = myAgent.extractContent(msg);
                finished = true;
                myAgent.addBehaviour(new DoGenerationBehavior(myAgent));

            } else {
                output.println("\nAgent " + myAgent.getLocalName() + " has not received message: " + message);
                block();
            }
//    } // End of try
//    catch (FIPAException fe) {
//            System.err.println("FIPAException in fill/extract content:" + fe.getMessage());
//            System.exit(-1);
//    }
        }

        public boolean done() {
            return finished;
        }


    } // end WaitForAnalysisComplete

    class DoGenerationBehavior extends SimpleBehaviour {
        private boolean finished = false;

        public DoGenerationBehavior(Agent a) {
            super(a);
        }

        public void action() {
            if (generator.generate())
                myAgent.addBehaviour(new RunsCompleted(myAgent));
            else
                myAgent.addBehaviour(new RequestEvaluateBehavior(myAgent));
            finished = true;
        }

        public boolean done() {
            return finished;
        }
    } // end DoGenerationBehavior


    class RunsCompleted extends SimpleBehaviour {
        private boolean finished = false;

        public RunsCompleted(Agent a) {
            super(a);
        }

        public void action() {
            output.println("\nAgent " + myAgent.getLocalName() + "  is in RunsCompleted");
            System.out.println("\nAgent " + myAgent.getLocalName() + "  is RunsCompleted");
            sendRunsCompletedMessage(OldMcDataAgent.evaluator);
            sendRunsCompletedMessage(OldMcDataAgent.starter);
            finished = true;
            output.close();
            output.println("\nAgent " + myAgent.getLocalName() + " is outa here!!");
            System.out.println("\nAgent " + myAgent.getLocalName() + " is outa here!!");
            myAgent.doDelete();
        }

        private void sendRunsCompletedMessage(String agentLocalName) {
            AID r = new AID();
            r.setLocalName(agentLocalName);
            ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
            msg.setSender(myAgent.getAID());
            msg.addReceiver(r);
            msg.setContent("RunsCompleted");
            msg.setLanguage("PlainText");
            msg.setOntology("RunsCompleted");
            send(msg);
        }

        public boolean done() {
            return finished;
        }
    } // end RunsCompleted
}











