/*
    OldMcData - The Data Farmer
    Copyright (C) 2005 Referentia Systems, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package oldmcdata.analyzers;

import java.text.NumberFormat;
import java.util.List;
import java.util.Vector;

import oldmcdata.analyzers.utils.DataSelector;
import oldmcdata.analyzers.utils.FileWatcher;
import oldmcdata.analyzers.utils.FitnessCalculator;
import oldmcdata.analyzers.utils.FitnessCalculatorFactory;
import oldmcdata.analyzers.utils.SimpleFitnessCalculator;
import oldmcdata.utils.StudyXML;

import org.dom4j.Element;

/**
 * @author Stephen C. Upton
 * @date Feb 14, 2005
 *
 */
public class AnotherFitnessAnalyzer extends AbstractAnalyzer {
    private final static String FITNESS = "Fitness",
    MOENAME = "MOEName",
    MOEPOSITION = "MOEPosition",
    STATISTIC = "Statistic",
    FITNESSCALCULATOR = "FitnessCalculator";
    private FitnessCalculator fitnessCalculator;
    private FileWatcher fileWatcher;
    private DataSelector dataSelector;
    private NumberFormat numberFormatter;

    public AnotherFitnessAnalyzer() {
        fitnessCalculator = new SimpleFitnessCalculator();
        fitnessCalculator.setStatistic("stats.Mean");
        fileWatcher = new FileWatcher();
        dataSelector = new DataSelector();
        numberFormatter = NumberFormat.getInstance();
        numberFormatter.setMaximumFractionDigits(3);
    }

    public void initialize() {
        System.out.println(this.getClass().getName() + ":initialize()");
        try {
            configure();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }
    }

    private void configure() {
        studyXMLHandler.openStudyXMLDocument();
        Element analyzerElement = studyXMLHandler.getAnalyzerAlgorithmElement();
        Element dataSelectorElement = analyzerElement.element("DataSelector");
        if ( dataSelectorElement == null ) {
            throw new NullPointerException("No DataSelector specified; can not process output");
        }
        dataSelector.initialize(dataSelectorElement);
        Element fitnessCalculatorElement = analyzerElement.element("FitnessCalculator");
        if ( fitnessCalculatorElement == null ) {
            System.out.println("No FitnessCalculator specified; using SimpleFitnessCalculator");
        }
        else {
            String fitnessClassName = fitnessCalculatorElement.elementText("classname");
            if ( fitnessClassName == null ) {
                System.out.println("No FitnessCalculator specified; using SimpleFitnessCalculator");
            }
            else {
                fitnessCalculator = FitnessCalculatorFactory.getInstance(fitnessClassName);
                fitnessCalculator.initialize(fitnessCalculatorElement);
            }
        }
    }


    public boolean analyze() {
        System.out.println(this.getClass().getName() + ":analyze()");
        studyXMLHandler.openStudyXMLDocument();
        if (!checkOutputFiles()) return false;
        processExcursions();
        studyXMLHandler.writeStudyXMLDocument();
        return true;
    }

    private boolean checkOutputFiles() {
        fileWatcher.setMoeFilesToWatch(studyXMLHandler.getMOEFileNames());
        return fileWatcher.checkForOutput();
    }

    private void processExcursions() {
        Element iterationElement = studyXMLHandler.getMostRecentIterationElement();
        List moeFileNames = studyXMLHandler.getMOEFileNames(iterationElement);

        List excursions = iterationElement.elements(StudyXML.EXCURSION);
        for (int i = 0; i < excursions.size(); i++) {
            processExcursion((Element) excursions.get(i), (String)moeFileNames.get(i));
        }

    }

    private void processExcursion(Element excursion, String moeFileName) {
        String excursionNumber = excursion.attributeValue(StudyXML.ID);
        System.out.println("Analyzing excursion: " + excursionNumber);
        Element fitness = excursion.addElement(FITNESS);
        dataSelector.setFileToProcess(moeFileName);
        dataSelector.processData();
        Vector colData = dataSelector.getColumns();

        Double realFitness = fitnessCalculator.calculateFitness(colData);
        fitness.setText(numberFormatter.format(realFitness.doubleValue()));
        System.out.println("Fitness: " + realFitness.toString());
    }


}
