/*
    
    oldmcdata1.0
    Copyright (C) 2005 Referentia Systems, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package oldmcdata.analyzers.utils;

import java.util.Vector;

import oldmcdata.analyzers.utils.stats.Statistic;
import oldmcdata.analyzers.utils.stats.StatisticFactory;

import org.dom4j.Element;

/**
 * @author Stephen C. Upton
 * @date Feb 22, 2005
 *
 */
public class ForceExchangeRatio implements FitnessCalculator {
    private Statistic statistic;
    private int numeratorCol, denominatorCol;
    
    public ForceExchangeRatio() {
        statistic = StatisticFactory.getInstance("Mean");  //default
    }
    
    public void setStatistic(String stat) {
        statistic = StatisticFactory.getInstance(stat);
    }

    public Double calculateFitness(double[] data){
        statistic.calculateStatistic(data);

        return statistic.getStatistic();
    }

    public Double calculateFitness(String stat, double[] data){
        setStatistic(stat);

        return calculateFitness(data);
    }
    
    // data is a Vector of DataVectors
    public Double calculateFitness(Vector data){
        if ( data.size() != 2 )
            throw new IllegalArgumentException("this fitness calculator requires 2 columns of data");
        
        
        DataVector numerator = (DataVector)data.get(numeratorCol-1);
        double[] numeratorValues = numerator.getValues();
        DataVector denominator = (DataVector)data.get(denominatorCol-1);
        double[] denominatorValues = denominator.getValues();
        int numberOfRowsOfData = numerator.getSize();
        double[] values = new double[numberOfRowsOfData];
        for (int i=0;i<numberOfRowsOfData;i++) {
            values[i] = (1.0 + numeratorValues[i])/(1.0 + denominatorValues[i]);
        }
        
        return calculateFitness(values);
    }
    
    public void initialize(Element e){
        String numeratorValue = e.elementText("numerator");
        String denominatorValue = e.elementText("denominator");
        numeratorCol = Integer.parseInt(numeratorValue);
        denominatorCol = Integer.parseInt(denominatorValue);
    }
}
