package oldmcdata.analyzers.utils;

import java.io.File;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class OutputSchemaBuilder {
    private final static String DEFAULTDATADELIMITER = ",",
    MOEFILEFORMAT = "MOEFileFormat",
    HEADER = "header",
    DELIMITER = "Delimiter",
    VALUE = "value",
    MOE = "MOE",
    NAME = "name",
    POSITION = "position",
    TYPE = "type";
    private Map moeMap;
    private String delimiter, modelName, modelVersion;
    private Document configDocument;
    private Element moeFileFormat;
    private File configFile;

    public OutputSchemaBuilder() {
        moeMap = new HashMap();
        delimiter = DEFAULTDATADELIMITER;
    }

    public void setConfigFile(File cf) {
        configFile = cf;
    }

    public void setModelInfo(String modelName, String modelVersion) {
        this.modelName = modelName;
        this.modelVersion = modelVersion;
    }

    public void buildSchema() {
        openConfigFile();
        setMOEFileFormatElement();
        setDelimiter();
        buildDataMap();
    }

    public void buildSchema(String headerline) {
        buildDataMap(headerline);
    }

    public void buildSchema(String moe, String moePosition) {
        moeMap.put(moe, new Integer(moePosition));
    }

    private void openConfigFile() {
        try {
            SAXReader xmlReader = new SAXReader();
            xmlReader.setStripWhitespaceText(true);
            configDocument = xmlReader.read(configFile);
        } 
//        catch (MalformedURLException mue) {
//            System.out.println("Can not form URL from this file: " + configFile);
//            mue.printStackTrace(System.out);
//            System.exit(-1);
//        } 
        catch (DocumentException de) {
            System.out.println("Error attempting to parse: " + configFile);
            de.printStackTrace(System.out);
            System.exit(-1);
        }
    }

    private void setMOEFileFormatElement() {
        Element configModelElement = null;
		StringBuffer xpathString = new StringBuffer("/OldMcDataConfig/Model[@name='");
		xpathString.append(modelName).append("' and @version='");
		xpathString.append(modelVersion).append("']");
		XPath xpath = DocumentHelper.createXPath(xpathString.toString());
//        XPath xpath = DocumentHelper.createXPath("/OldMcDataConfig/Model[@name=$name and @version=$version]");
//        SimpleVariableContext svc = new SimpleVariableContext();
//        svc.setVariableValue("name", modelName);
//        svc.setVariableValue("version", modelVersion);
//        xpath.setVariableContext(svc);
        Node modelNode = xpath.selectSingleNode(configDocument.getRootElement());
        if (modelNode == null) {
            throw new RuntimeException("OLDMCDATA FAILED: Can not locate model/version " +
                    modelName + "-" + modelVersion + " in the configFile: " +
                    configFile.toString());
        } else
            configModelElement = (Element) modelNode;
        moeFileFormat = configModelElement.element(MOEFILEFORMAT);
    }

    public void setBasecaseFileInfo(File studyDirectory, String basecaseFileName) {

    }

    private void setDelimiter() {
        Element delimiterElement = moeFileFormat.element(DELIMITER);
        if (delimiterElement != null) {
            delimiter = delimiterElement.attributeValue(VALUE);
        }
    }

    private void buildDataMap() {
        List moes = moeFileFormat.elements(MOE);
        Iterator its = moes.iterator();
        while (its.hasNext()) {
            Element moe = (Element) its.next();
            String moeName = moe.attributeValue(NAME);
            String column = moe.attributeValue(POSITION);
            moeMap.put(moeName, new Integer(column));
        }

    }

    private void buildDataMap(String headerLine) {
        StringTokenizer tokenizer = new StringTokenizer(headerLine, delimiter);
        int position = 0;
        while (tokenizer.hasMoreTokens()) {
            moeMap.put(tokenizer.nextToken(), new Integer(position));
            position++;
        }
    }

    public Map getMOEMapping() {
        return moeMap;
    }

    public String getDelimiter() {
        return delimiter;
    }

    public int getMOEPosition(String moeName) {
        Object column = moeMap.get(moeName);
        if (column == null)
            throw new RuntimeException("MOE: " + moeName + " does not match any MOE in " +
                    configFile.toString());

        return ((Integer) column).intValue();
    }

    public void printMOEMapping(PrintStream out) {
        Iterator it = moeMap.entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry entry = (Map.Entry) it.next();
            String name = (String) entry.getKey();
            Integer col = (Integer) entry.getValue();
            out.println("MOE: " + name + " Column: " + col.toString());
        }
    }

}