package oldmcdata.evaluators.condor;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.dom4j.Element;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */


public class CondorSubmitFileCreator {
    private final static String space = " ",
    DEFAULTENTRYSEPARATOR = "=";
    private File submitFile;
    private Element condorElement;
    private List condorLines;
    private String lineSeparator;
    private int currentIteration;

    public CondorSubmitFileCreator() {
        lineSeparator = System.getProperty("line.separator");
        condorLines = new ArrayList();
    }

    public void createSubmitFile(File studyDirectory, Element condorElement, int i) {
        condorLines.clear();
        currentIteration = i;
        this.condorElement = condorElement;
        List condorEntries = condorElement.elements();
        for (int j = 0; j < condorEntries.size(); j++) {
            Element condorEntry = (Element) condorEntries.get(j);
            condorLines.add(buildLine(condorEntry));
        }
        submitFile = new File(studyDirectory, "submit-" + i + ".dat");
        writeSubmitFile();
    }

    private String buildLine(Element condorEntry) {
        StringBuffer entry = new StringBuffer(condorEntry.getName());
        String separator = condorEntry.attributeValue("sep");
        if (separator == null) separator = DEFAULTENTRYSEPARATOR;
        entry.append(space).append(separator).append(space);
        String type = condorEntry.attributeValue("type");
        if (type == null) {
            buildSimpleEntry(entry, condorEntry);
            return entry.toString();
        }
        if (type.equalsIgnoreCase("list"))
            buildListEntry(entry, condorEntry);
        if (type.equalsIgnoreCase("iteration"))
            buildIterationEntry(entry, condorEntry);

        return entry.toString();
    }

    private void buildListEntry(StringBuffer entry, Element condorEntry) {
        String argsep = getArgumentSep(condorEntry);
        List elements = condorEntry.elements();
        for (int i = 0; i < elements.size() - 1; i++) {
            Element e = (Element) elements.get(i);
            addPrefix(entry, e);
            entry.append(e.getText());
            addSuffix(entry, e);
            entry.append(argsep);
        }
        Element lastone = (Element) elements.get(elements.size() - 1);
        buildSimpleEntry(entry, lastone);
    }

    private String getArgumentSep(Element condorEntry) {
        String argsep = condorEntry.attributeValue("argsep");
        if (argsep == null)
            return space;
        String argsepOp = condorEntry.attributeValue("argsepOp");
        if (argsepOp == null || argsepOp.equalsIgnoreCase("false"))
            return argsep;
        else {
            if (argsep.equalsIgnoreCase("OR"))
                return " || ";
            if (argsep.equalsIgnoreCase("AND"))
                return " && ";
        }
        return argsep;
    }

    private void buildSimpleEntry(StringBuffer entry, Element condorEntry) {
        addPrefix(entry, condorEntry);
        entry.append(condorEntry.getText());
        addSuffix(entry, condorEntry);
    }

    private void buildIterationEntry(StringBuffer entry, Element condorEntry) {
        addPrefix(entry, condorEntry);
        entry.append(condorEntry.getText());
        entry.append(Integer.toString(currentIteration));
        addSuffix(entry, condorEntry);
    }

    private void addPrefix(StringBuffer entry, Element e) {
        String prefix = e.attributeValue("prefix");
        if (prefix == null)
            return;
        else
            entry.append(prefix);
    }

    private void addSuffix(StringBuffer entry, Element e) {
        String suffix = e.attributeValue("suffix");
        if (suffix == null)
            return;
        else
            entry.append(suffix);
    }

    public String getSubmitFile() {
        return submitFile.toString();
    }

    public void writeSubmitFile() {
        try {
            FileWriter fw = new FileWriter(submitFile);
            for (int i = 0; i < condorLines.size(); i++) {
                String condorLine = (String) condorLines.get(i);
                fw.write(condorLine);
                fw.write(lineSeparator);
            }
            fw.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + submitFile);
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
    }

}

