package oldmcdata.evaluators.local;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import oldmcdata.evaluators.AbstractCommandLineBuilder;
import oldmcdata.utils.StudyXMLHandler;

import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */


public class LocalCommandLineBuilder extends AbstractCommandLineBuilder {
    private final static String TRANSFERINPUTFILES = "transfer_input_files";
    protected File excursionDirectory, playBackDirectory, studyDirectory;
    private File executableFile, excursionFile, moeFile, playbackFile;
    private Iterator excurFileIt, moeFileIt, playbackFileIt;
    private List<LocalJob> localJobs;
    private List<File> filesToTransfer;
    private String commandPrefix;
    private String commandPrearg;

    public LocalCommandLineBuilder() {
        localJobs = new ArrayList<LocalJob>();
        filesToTransfer = new ArrayList<File>();
    }

    public void buildCommandLines(File configFile, StudyXMLHandler studyXMLHandler,
                                  Map argumentMap, int numberOfExcursions) {
        this.configFile = configFile;
        this.studyXMLHandler = studyXMLHandler;
        this.argumentMap = argumentMap;
        this.numberOfExcursions = numberOfExcursions;

        try {
            openStudyXML();
            openConfigFile();
            setModelElement();
            setParameters();
            setReplicationsInfo();
            setPlaybacksWanted();
            buildCommandLines();
        } catch (Exception e) {
            e.printStackTrace(System.out);
            System.exit(-1);
        }
    }

    @SuppressWarnings("unchecked")
	protected void setParameters() {
    	studyDirectory = studyXMLHandler.getStudyDirectory();
        excursionDirectory = studyXMLHandler.getExcursionDirectory();
        playBackDirectory = studyXMLHandler.getPlaybackDirectory();
        executableElement = getExecutableElement(COMMAND);
        if (executableElement == null)
            throw new RuntimeException("OLDMCDATA ERROR: can not find matching executable in " +
                    configFile.toString());
        executable = executableElement.attributeValue(VALUE);
        executableFile = new File(executable);
        if (executable == null)
            throw new RuntimeException("OLDMCDATA ERROR: no executable specified in " +
                    configFile.toString());
        filesToTransfer.add(executableFile);
        
        commandPrefix = executableElement.attributeValue(PREFIX);
        commandPrearg = executableElement.attributeValue("prearg");
        
        argumentsElement = executableElement.element(ARGUMENTS);
        // if arguments are not specified in the config file, then just use defaults of
        // false for all of the parameters
        if (argumentsElement == null)
            throw new RuntimeException("OLDMCDATA ERROR: an ordering of arguments in " +
                    configFile.toString() +
                    " has not been specified");
        
		// add files that are specific to the model
        Element modelTransferInputFiles = executableElement.element(TRANSFERINPUTFILES);
        List<Element> files = modelTransferInputFiles.elements("file");
        for (Element fileToTransfer : files) {
        	filesToTransfer.add(new File(fileToTransfer.getTextTrim()));
        }
        
        //always transfer the study.xml file
        filesToTransfer.add(studyXMLHandler.getStudyFile());
        
        // add in mapfiles from study.xml
        Element modelParameters = studyXMLHandler.getModelParametersElement();
        List<Element> mapFiles = modelParameters.elements("MapFile");
        for (Element mapFile : mapFiles) {
            filesToTransfer.add(getMapFileLocation(mapFile));
        }

    }
    
    /** Sets MapFileLocation to location of the MapFile specified in the study file. If MapFile is a file name,
     * assume MapFile is in study directory. This method will then append the study directory
     * to the MapFile name. If the MapFile includes a directory, then assume the
     * MapFile is in that absolute directory location.
     * @param file
     * @return
     */
    private File getMapFileLocation(Element file) {
        String mapFileName = file.getTextTrim();
        File mapFile = new File(mapFileName);
        String mapFileDir = mapFile.getParent();
        if (mapFileDir == null) { // then assume MapFile is in study directory
            mapFile = new File(studyDirectory, mapFileName);
        }
        if (!mapFile.exists())
            throw new RuntimeException("Can't locate MapFile: " + mapFile.toString());

        return mapFile;
    }

    protected void buildCommandLines() {
        try {
			commandLines = new ArrayList();
			// the list of elements is now whatever is in the argumentElements
			// items could have been removed by the previous methods
			argumentList = argumentsElement.elements();
			excurFileIt = (Iterator) argumentMap.get("ExcursionFile");
			moeFileIt = (Iterator) argumentMap.get("MoeFile");
			playbackFileIt = (Iterator) argumentMap.get("PlaybackFile");

			for (int i = 0; i < numberOfExcursions; i++) {
			    // construct an individual job for each excursion
				createLocalJob(i);
			}
		} catch (NullPointerException e) {
			e.printStackTrace();
			System.out.println("In CommandLineBuilder::buildCommandLines");
			System.exit(-1);
		}

    }
    
    public List<LocalJob> getLocalJobs() {
    	return localJobs;
    }
    
    private void createLocalJob(int excNum) {
    	// get ExcursionFile name
        excursionFile = (File) excurFileIt.next();
        System.out.println("excursion path: " + excursionFile.getPath());
        moeFile = (File) moeFileIt.next();
        playbackFile = (File) playbackFileIt.next();
		LocalJob job = new LocalJob(buildCommandLine());
		// add all the model and study files to transfer common for each job
		for (File fileToTransfer : filesToTransfer) {
			job.addFileToTransfer(fileToTransfer);
		}
		job.addFileToTransfer(excursionFile);
		localJobs.add(job);

    }
    
    /* The command line is local to the run directory, so all filenames should only refer to
     * the file name last in the path sequence.
     * 
     */
    @SuppressWarnings("unchecked")
	private List<String> buildCommandLine() {
        ArrayList<String> commandLine = new ArrayList<String>();
        if ( commandPrefix != null ) {
        	commandLine.add(commandPrefix);
        	if ( commandPrearg != null) {
        		commandLine.add(commandPrearg);
        	}
        }
        
        commandLine.add(executableFile.getName());
        for (int i=0;i<argumentList.size();i++) {
        	Element argument = (Element)argumentList.get(i);
        	String argumentValue = argument.getTextTrim();
            if (argumentValue.equalsIgnoreCase("ExcursionFile")) {
                commandLine.add(excursionFile.getName());
                continue;
            }
            if (argumentValue.equalsIgnoreCase("NumberOfReplications")) {
            	commandLine.add(numberOfReplications);
                continue;
            }
            if (argumentValue.equalsIgnoreCase("MoeFile")) {
                commandLine.add(moeFile.getName());
                continue;
            }
            if (argumentValue.equalsIgnoreCase("Seed")) {
                String seed = (String) seedList.get(0);
                commandLine.add(seed);
                continue;
            }
            if (argumentValue.equalsIgnoreCase("PlaybackFile")) {
                commandLine.add(playbackFile.getName());
                continue;
            }

            //otherwise, take argument literally
            commandLine.add(argumentValue);
        }
        commandLines.add(commandLine);
        return commandLine;
    }

    
}

