package oldmcdata.evaluators.local;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import oldmcdata.evaluators.AbstractEvaluator;
import oldmcdata.evaluators.CommandBuilder;
import oldmcdata.utils.CopyFilesFromTo;
import oldmcdata.utils.ExcursionFileCreator;

import org.dom4j.Element;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */

public class LocalEvaluator extends AbstractEvaluator {
    private final static String MAKERUNS = "MakeRuns";
    private LocalJobRunner jobRunner;
    private ExcursionFileCreator excursionFileCreator;
    private boolean makeRuns = true;
    private LocalCommandLineBuilder cmdLineBuilder;
    private String[] cmdarray;
    private File studyDirectory;

    public LocalEvaluator() {
        super();
        jobRunner = new LocalJobRunner();
        excursionFileCreator = new ExcursionFileCreator();
        cmdLineBuilder = new LocalCommandLineBuilder();
    }

    public void initialize() {
        System.out.println(this.getClass().getName() + " is in initialize()");
        try {
            configure();
        } catch (Exception e) {
            e.printStackTrace(System.out);
            System.exit(-1);
        }
    }

    public void setMakeRuns(String noe) {
        Boolean b = Boolean.valueOf(noe);
        makeRuns = b.booleanValue();
    }

    private void configure() {
        Element parameters = studyXMLHandler.getEvaluatorParameters();
        Element makeRunsElement = parameters.element(MAKERUNS);
        if (makeRunsElement != null)
            setMakeRuns(makeRunsElement.getTextTrim());
        // else do nothing makeRuns remains true - false setting used for testing
        if (!configFile.exists())
            throw new RuntimeException("OLDMCDATA FAILED:OLDMCDATA configuration file: " +
                    configFile.toString() + " does not exist");
        System.out.println("configFile used: " + configFile.toString());
    }

    @SuppressWarnings("unchecked")
	public void evaluate() {
        System.out.println(this.getClass().getName() + " is in evaluate()");
        studyDirectory = studyXMLHandler.getStudyDirectory();
        excursionFileCreator.createExcursions(studyXMLHandler);
        cmdLineBuilder.buildCommandLines(configFile, studyXMLHandler,
                excursionFileCreator.getArgumentMap(),
                excursionFileCreator.getNumberOfExcursions());
        if (makeRuns) {
        	// create and run local jobs
        	jobRunner.setStudyDirectory(studyDirectory);
        	jobRunner.setOutputDirectory(studyXMLHandler.getMOEDirectory());
            jobRunner.runLocalJobs(cmdLineBuilder.getLocalJobs());
        } else {
            List commandLines = cmdLineBuilder.getCommandLines();
            System.out.println("CommandLines");
            for (int i = 0; i < commandLines.size(); i++) {
                List commandLine = (List) commandLines.get(i);
                for (int j = 0; j < commandLine.size(); j++) {
                    String arg = (String) commandLine.get(j);
                    System.out.print(arg);
                    System.out.println();
                }
            }
        }
    }
    
    
    private void transferFiles() {
    	transferMapFiles();
    }

    /*Transfers the files listed under the MapFile element of the study.xml file
     * to the Excursion directory. Files in the MapFile element are either assumed
     * to be in the studyDirectory, given an absolute path or a relative path.
     * 
     */
    private void transferMapFiles() {
        // transfer mapfiles to Excursion Directory
        File excursionDir = studyXMLHandler.getExcursionDirectory();
        Element modelParameters = studyXMLHandler.getModelParametersElement();
        List mapFiles = modelParameters.elements("MapFile");
        for (Iterator maps = mapFiles.iterator(); maps.hasNext();) {
            Element file = (Element) maps.next();
            File mapFile = new File(file.getTextTrim());
            System.out.println("Attempting to copy  " + mapFile.toString() + " to Excursion Directory");
            copyFile(mapFile, excursionDir);
        }

    }

    /* Copy fileToCopy from current location to same filename in
     * the directoryToCopyTo directory. If fileToCopy has no parent,
     * assumes that the fileToCopy is in the studyDirectory.
     */
    private void copyFile(File fileToCopy, File directoryToCopyTo) {
    	String fileName = fileToCopy.getName(); // filename in directory
    	File destFile = new File(directoryToCopyTo, fileName);
    	File srcFile = new File(fileToCopy.getPath()); // get the full path to the file
    	String parent = fileToCopy.getParent();
    	if (parent == null) // assumes fileToCopy is in studyDirectory, i.e.
    						// no parent implies a relative path name was used in the study.xml file
    		srcFile = new File(studyXMLHandler.getStudyDirectory(), fileToCopy.toString());
        CopyFilesFromTo.copyFromTo(srcFile, destFile);
    }

}

