/**
 * 
 */
package oldmcdata.evaluators.local;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * LocalJob holds information about a job to be run on the local machine.
 * 
 *Copyright: 2010
 *  
 * @author Stephen C. Upton
 * @version 1.1
 */
public class LocalJob implements Runnable {
	private static int count;
	private int id;
	private List<String> commandLine;
	private List<File> transferFiles;
	private String outFileName = "job";
	private FileWriter fw = null;
	private File runDirectory;
	
	public LocalJob(List<String> commandLine) {
		this.id = count++;
		this.commandLine = commandLine;
		transferFiles = new ArrayList<File>();

	}
	
	public int getJobID() {
		return id;
	}
	
	public void addFileToTransfer(File f) {
		transferFiles.add(f);
	}
	
	public List<File> getInputTransferFiles() {
		return transferFiles;
	}
	
	public void setRunDirectory(File runDir) {
		runDirectory = runDir;
	}

	public void run() {
		openOutFile();
		runCommandLine();
		closeOutFile();
		
	}
	
	public void openOutFile() {
        try {
            fw = new FileWriter(new File(runDirectory, outFileName + "." + id + ".log"));
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + fw.toString());
            ioe.printStackTrace(System.out);
        }		
	}
	
	public void closeOutFile() {
        try {
            fw.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to close: " + fw.toString());
            ioe.printStackTrace(System.out);
        }			
	}
	
    public void runCommandLine() {
    	long startTime = System.currentTimeMillis();
        System.out.println("Command Line: ");
        for (String cmd : commandLine ) 
            System.out.print(cmd+ " ");
        System.out.println();
   		
        try {
        	ProcessBuilder pb = new ProcessBuilder(commandLine);
        	pb.directory(runDirectory);
        	System.out.println("rundir: " + pb.directory().toString());
        	Process pp = pb.start();
        	BufferedReader results = new BufferedReader(
        			new InputStreamReader(pp.getInputStream()));
        	String s;
        	while ((s = results.readLine()) != null) {
        		fw.write(s);
        		fw.write("\n");
        	}
        		
        	BufferedReader errors = new BufferedReader(
        			new InputStreamReader(pp.getErrorStream()));
        	while ((s = errors.readLine()) != null ) {
        		System.err.println(s);
        	}
        	pp.waitFor();
        	long currentTime = System.currentTimeMillis();
        	System.out.println("Time for Job ID: " + id + ": " 
        			+ startTime + ": " + currentTime + ": " 
        			+ Long.toString(currentTime - startTime));

        } catch (Exception ex) {
            ex.printStackTrace(System.out);
            System.exit(-1);
        }


    }	
	
}
