package oldmcdata.generators;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import oldmcdata.utils.DefaultTranslator;
import oldmcdata.utils.StudyXML;
import oldmcdata.utils.StudyXMLHandler;
import oldmcdata.utils.Translator;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;


/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.3
 */

/** A class to test implementation of Generators
 */
public abstract class AbstractGenerator implements Generator {
    protected StudyXMLHandler studyXMLHandler;
    protected Translator translator;
    protected File configFile;

    public AbstractGenerator() {
        translator = new DefaultTranslator();
    }

    public void setStudyXMLHandler(StudyXMLHandler e) {
        studyXMLHandler = e;
    }

    public void setConfigFile(String filename) {
        configFile = new File(filename);
    }

    public abstract void initialize();

    public abstract boolean generate();

    public void writeRenamedBaseCase() {
        Element iteration = studyXMLHandler.getMostRecentIterationElement();
        Element generatorAlgorithm = studyXMLHandler.getGeneratorAlgorithmElement();
        Element iterationExcursionFileInfo = iteration.element(StudyXML.EXCURSIONFILEINFO);
        Element basicExcursionInfo = generatorAlgorithm.element(StudyXML.EXCURSIONFILEINFO);

        String basecaseFileName = basicExcursionInfo.element(StudyXML.BASECASEFILENAME).getText();
        String renamedBaseCaseFileName = iterationExcursionFileInfo.element(StudyXML.BASECASEFILENAME).getText();
        File basecaseFile = new File(studyXMLHandler.getStudyDirectory(), basecaseFileName);
        try {
            SAXReader xmlReader = new SAXReader();
            xmlReader.setStripWhitespaceText(false);
            Document basecaseDoc = xmlReader.read(basecaseFile);
            File renamedBaseCaseFile = new File(studyXMLHandler.getStudyDirectory(), renamedBaseCaseFileName);
            writeCleanBaseCaseXMLDocument(renamedBaseCaseFile, basecaseDoc);

        } 
//        catch (MalformedURLException mue) {
//            System.out.println("Can not form URL from this file: " + basecaseFile);
//            mue.printStackTrace(System.out);
//            System.exit(-1);
//        } 
        catch (DocumentException de) {
            System.out.println("Error attempting to parse: " + basecaseFile);
            de.printStackTrace(System.out);
            System.exit(-1);
        }

    }

    /** Cleans up the basecase document by stripping off leading and trailing whitespace. We do this
     so that XPath will work correctly when the basecase doc has leading spaces in variables that are used
     to select specific variables within an element. For example, if we desire to change the Sensor tag, with a name
     of "BlueSensor", then " BlueSensor " will work also. Writes this temporary
     document to the file indicated by temp.
     */
    public void writeCleanBaseCaseXMLDocument(File temp, Document basecaseDoc) {
        try {
            FileWriter fw = new FileWriter(temp);
            OutputFormat of = OutputFormat.createCompactFormat();
            of.setTrimText(false);
            XMLWriter xmlWriter = new XMLWriter(fw, of);
            xmlWriter.write(basecaseDoc);
            xmlWriter.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to open: " + temp.toString());
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
    }

}
