package oldmcdata.generators;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import oldmcdata.utils.StudyXML;

import org.dom4j.Element;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.3
 */

/** A class to test implementation of Generators
 */
public class GriddedGenerator extends AbstractGenerator {
    private final static String DELIMITER = "Delimiter",
    DEFAULTDELIMITER = ",",
    MAXIMUMVALUE = "MaximumValue",
    MINIMUMVALUE = "MinimumValue",
    DELTAVALUE = "Delta",
    INTEGER = "integer",
    FLOAT = "float";
    private List excursions;
    private String delim;
    private int[] currentIndices;
    private int[] maxIndices;
    private List dataList;
    private int numberOfExcursions = 1;
    private boolean generatedExcursions = false;

    public GriddedGenerator() {
        super();
        delim = DEFAULTDELIMITER;
        dataList = new ArrayList();
    }

    public void initialize() {
        System.out.println(this.getClass().getName() + ":initialize()");
        translator.initialize();
        configure();
    }

    private void configure() {
        Element parameters = studyXMLHandler.getGeneratorAlgorithmParameters();
        Element delimiter = parameters.element(DELIMITER);
        if (delimiter != null)
            setDelimiter(delimiter.getText());
    }

    public void setDelimiter(String d) {
        delim = d;
    }

    public boolean generate() {
        if (!generatedExcursions) {
            System.out.println(this.getClass().getName() + ":generate()");
            studyXMLHandler.openStudyXMLDocument();
            calculateGrid();
            generateExcursions();

            translator.translate(excursions, studyXMLHandler);
            studyXMLHandler.writeStudyXMLDocument();
            writeRenamedBaseCase();
            generatedExcursions = true;
            return false;
        }

        return true;
    }

    public void calculateGrid() {
        List variables = studyXMLHandler.getVariables();
        for (Iterator var = variables.iterator(); var.hasNext();)
            calculateGridValues((Element) var.next());

        currentIndices = new int[dataList.size()];
        maxIndices = new int[dataList.size()];
        for (int i = 0; i < currentIndices.length; i++) {
            currentIndices[i] = 0;
            maxIndices[i] = ((ArrayList) dataList.get(i)).size();
            numberOfExcursions *= maxIndices[i];
        }
    }

    private void calculateGridValues(Element variable) {
        ArrayList parameterList = new ArrayList();
        dataList.add(parameterList);
        String type = variable.attributeValue(StudyXML.TYPE);

        Double maximumValue = new Double(variable.element(MAXIMUMVALUE).getText());
        Double minimumValue = new Double(variable.element(MINIMUMVALUE).getText());
        Double deltaValue = new Double(variable.element(DELTAVALUE).getText());
        double value = minimumValue.doubleValue();
        double upperBounds = maximumValue.doubleValue();
        double delta = deltaValue.doubleValue();
        while (value <= upperBounds) {
            if (type.equalsIgnoreCase(INTEGER))
                parameterList.add(new Integer((int) Math.round(value)).toString());
            if (type.equalsIgnoreCase(FLOAT))
                parameterList.add(Double.toString(value));
            value += delta;
        }

    }

    private void incrementCurrentIndices() {
        for (int i = 0; i < currentIndices.length; i++) {
            ++currentIndices[i];
            if (currentIndices[i] >= maxIndices[i]) {
                currentIndices[i] = 0;
                continue;
            } else
                break;
        }
    }

    private void generateExcursions() {
        excursions = new ArrayList();
        for (int i = 0; i < numberOfExcursions; i++) {
            excursions.add(generateExcursion());
            incrementCurrentIndices();
        }
    }

    private String generateExcursion() {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < currentIndices.length; i++) {
            ArrayList values = (ArrayList) dataList.get(i);
            String valueToAdd = (String) values.get(currentIndices[i]);
            sb.append(valueToAdd).append(delim);
        }
        sb.deleteCharAt(sb.length() - 1);
        return sb.toString();
    }

}
