package oldmcdata.generators.utils;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

import nalex.math.util.RealVector;
import nalex.operators.evaluate.BasicEvaluate;
import nalex.populations.Individual;
import nalex.populations.Population;
import nalex.problem.Fitness;
import nalex.problem.Point;
import nalex.problem.Problem;

/**
 * Title:        Simulation/Model Search using Natural Algorithms
 * Description:  Using Natural Algorithms, we seek to find the set of "best" simulation output.
 *Copyright: 2000 - 2005
 *         
 * @author Stephen C. Upton
 * @version 1.0
 *
 * Used as link between NALEX algorithms and OLDMCDATA. Basic operator that just uses
 * problemType and fitness from OLDMCDATA evaluation to set fitness for NALEX individual
 */

public class NALEXModelEvaluate extends BasicEvaluate {
    private String[] pointStrings;
    private PrintWriter testData;
    private double fitnessValue;

    public NALEXModelEvaluate(String keyName) {
        super(keyName);
    }

    public NALEXModelEvaluate(String keyName, Problem p) {
        super(keyName);
        setProblem(p);
    }

    private void openTestFile() {
        try {
            testData = new PrintWriter(new BufferedWriter(
                    new FileWriter("test.txt", true)));
        } catch (IOException e) {
            e.printStackTrace(System.out);
        }
    }

    private void writeLine(String line) {
        testData.println(line);
    }

    private void closeTestFile() {
        testData.close();
    }

    public void updateFitnesses(Population pop, List fitnesses) {
        openTestFile();

        for (int i = 0; i < pop.size(); i++) {
            Individual nalexIndividual = pop.getIndividual(i);
            fitnessValue = Double.parseDouble((String) fitnesses.get(i));
            evaluate(nalexIndividual);
            writeLine("Nalex: " + nalexIndividual.toString());
        }

        closeTestFile();
    }

    /**
     *
     */
    public void evaluate(Individual ind) {
        Problem problem = this.getProblem();
        Point point = ind.getPoint();
        Fitness fitness = new Fitness(point, problem.getNumberOfObjectives());
        RealVector functionValue = new RealVector(1,fitnessValue);

        fitness.setFeasible(true);
        if (problem.getProblemType() == Problem.MIN)
            functionValue.scalarMultiplyThisBy(-1.0);

        fitness.setFitness(functionValue);
        ind.setFitness(fitness);
    }

}

