package oldmcdata.generators.utils;

import java.util.ArrayList;
import java.util.List;

import nalex.math.util.RealVector;
import nalex.problem.Function;
import nalex.problem.Point;
import nalex.problem.RealPoint;
import nalex.problem.UnconstrainedProblem;
import nalex.problem.exceptions.UnsupportedPointException;

import org.dom4j.Element;

/**
 * Title: NALEXModelProblem
 * Description: Interface between NALEX and OLDMCDATA software for problem
 specification
 *Copyright: 2000 - 2005
 *  	 
 * @author Stephen C. Upton
 * @version 1.0
 */

public class NALEXModelProblem extends UnconstrainedProblem {
    protected final static String MAXIMUMVALUE = "MaximumValue",
    MINIMUMVALUE = "MinimumValue";
    private List variables;
    private List dimensionInfo;

    public NALEXModelProblem(String problemType, List variables) {
        super("NALEXModelProblem");
        this.setDescription("NALEXModelProblem");
        this.variables = variables;
        setObjectiveFunction(new NALEXModelProblemFunction(variables));
        setProblemType(problemType);
        setDimension(getObjectiveFunction().getDimension());
        dimensionInfo = new ArrayList();
        setBounds();
    }

    public List getDimensionInformation() {
        return dimensionInfo;
    }

    public void setBounds() {
        RealVector lowerBounds = new RealVector(getDimension(), 0.0);
        RealVector upperBounds = new RealVector(getDimension(), 1.0);

        for (int i = 0; i < variables.size(); i++) {
            Element variable = (Element) variables.get(i);
            String variableName = variable.attributeValue("name");
            String maximumValue = variable.element(MAXIMUMVALUE).getText();
            String minimumValue = variable.element(MINIMUMVALUE).getText();
            upperBounds.setValue(i, Double.parseDouble(maximumValue));
            lowerBounds.setValue(i, Double.parseDouble(minimumValue));
            dimensionInfo.add(variableName);
        }

        setUpperBounds(upperBounds);
        setLowerBounds(lowerBounds);
    }
}

/** dummy Function to satisfy NALEX Problem interface
 *
 */
class NALEXModelProblemFunction extends Function {

    NALEXModelProblemFunction(List variables) {
        setDimension(variables.size());
        setName("counts number of dimensions");
    }

    public RealVector calculateFunctionValue(Point pt) throws UnsupportedPointException {
        if (!(pt instanceof RealPoint)) throw new UnsupportedPointException("NALEXModelProblemFunction only supports RealPoints");
        double f = 0.0;
        for (int i = 0; i < getDimension(); i++)
            f += 1.0;

        return new RealVector(1, f);
    }


}