/*
 * Created on May 4, 2004
 *
 */
package oldmcdata.postprocessors;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;



/**
 *Copyright: 2000 - 2005
 *   Referentia Systems, Inc.
 * @author Stephen C. Upton
 *
 */
public class DataReader {
	private List data;
	private BufferedReader reader;
	private PrintWriter logFileWriter;
	private RowSelector rowSelector;
	int numberOfLinesToSkip = 0;
	private PostProcessTask myTask;
	private int lineCount;
	private String extraInfoStartsWith;
	private Map extraInfoMap;
	
	public DataReader(PostProcessTask task) {
		myTask = task;
		this.configure();
	}
	
	public void configure() {
		this.logFileWriter = myTask.getLogFileWriter();
		rowSelector = new RowSelector(myTask, PostProcessTask.SKIPLINES);
		Map skipLinesMap = myTask.getDataMap(PostProcessTask.SKIPLINES);
		setNumberOfLinesToSkip((String)skipLinesMap.get(PostProcessTask.NUMBER));
		extraInfoMap = (Map)myTask.getDataMap(PostProcessTask.EXTRAINFO);
		extraInfoStartsWith = (String)extraInfoMap.get(PostProcessTask.STARTSWITH);
	}
	
	public List getData(File dataFile) {
		data = new ArrayList();
		if ( !openFile(dataFile) ) return data;
		
		logFileWriter.println("Begin processing file: " + dataFile.getName());
		
		readData();
		
		logFileWriter.println("Completed processing file: " + dataFile.getName());
		logFileWriter.println("Number of lines processed: " + lineCount);
		logFileWriter.println("Number of lines selected: " + data.size());
				
		return data;
	}

	private boolean openFile(File fileToOpen) {
		boolean result = true;
		try {
			reader = new BufferedReader(new FileReader(fileToOpen));
			 
		}
		catch(IOException ioe) {
			logFileWriter.println("Had trouble opening file: " + fileToOpen.getName());
			logFileWriter.println(ioe.getMessage());
			result = false;
		}

		return result;

	}

	private void readData() {
		lineCount = 0;
		String lineRead = "";
		List headerRows = myTask.getHeaderRows();
		boolean setHeaderRow = myTask.getUseHeaderRow();
		
		boolean eof = false;
		while (!eof) {
			try {
				lineRead = reader.readLine();
				lineCount++;
				if ( lineRead == null ) {
					eof = true;
					break;					
				}
				// first skip the number of lines specified by Number in SkipLines
				if ( lineCount <= numberOfLinesToSkip ) {
				    // first extract info, if specified in input specification
				    extractInfo(lineRead);
				    continue;
				}
				
				// assume next line after number of lines skipped is header row, if used
				if ( setHeaderRow ) {
					headerRows.add(lineRead);
					setHeaderRow = false;
					continue;
				}
			
				// now skip those that match the constraints
				if ( rowSelector.lineMatches(lineRead) ) {
				    // first extract info, if specified in input specification
				    extractInfo(lineRead);
				    continue;
				}
				// if the lineRead doesn't match any of the contraints, then add it	
				data.add(lineRead);

			}
			catch(IOException ioe) {
				logFileWriter.println("Had trouble reading line: " + lineCount);
				logFileWriter.println(lineRead);
			
			}			
		}
		
	}
	
	public void extractInfo(String lineRead) {
	    if ( extraInfoStartsWith == null ) return;
	    if ( lineRead.startsWith(extraInfoStartsWith)) {
	        extraInfoMap.put(PostProcessTask.VALUE,lineRead);
	        System.out.println("Extracting info from: " + lineRead);
	    }
	}

	
	public void setNumberOfLinesToSkip(String s){
		Integer number = new Integer(0);
		try {
			number = new Integer(s);
		}
		catch(NumberFormatException nfe) {
			logFileWriter.println("Unrecognized Number in SkipLines: " + s);
			logFileWriter.println("Using default of 0");
		}
		numberOfLinesToSkip = number.intValue();
	}


}
