/*
 * Created on May 1, 2004
 *
 */
package oldmcdata.postprocessors;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

/**
 *Copyright: 2000 - 2005
 *  @author Stephen C. Upton
 * 
 * modified by Steve Upton 01/13/05 with the following changes:
 * 	 -changed name to OMDPostProcessor, was OutputFilePostProcessor
 * 	 -removed reliance on Study.xml; this class now reads in postprocessor file or
 *      can be given an OMDPostProcessor XML element as a source for its PostProcessTasks
 *
 */
public class OMDPostProcessor {
	private String rootDirectory,postprocessInputFile;
	private List tasks;
	private File logFile;
	private PrintWriter logFileWriter;
	private Element postprocessorElement;

	public OMDPostProcessor() {
		//default no-argument construtor that can be used by the OMD classes
	}
	
	/** For use as stand-alone application for post processing
	 * 
	 * @param rootDirectory - directory containing output files or other subdirectories containing output files
	 * @param postprocessInputFile - XML input file specification for the PostProcessTasks
	 */
	public OMDPostProcessor(String rootDirectory, String postprocessInputFile) {
		this.rootDirectory = rootDirectory;
		this.postprocessInputFile = postprocessInputFile;
	}
	
	public void setPostProcessorRootElement(Element postprocessorElement) {
		this.postprocessorElement = postprocessorElement;
	}
	
	/** Used to set the root directory location so that the PostProcessTasks Directory
	 *  element has a path reference to the output files to process. We assume the output files 
	 * and directories containing the output are located below this root directory.
	 * @param rootDirectory
	 */
	public void setRootDirectory(String rootDirectory) {
		this.rootDirectory = rootDirectory;
	}
	
	public void openPostProcessInputFile() {
		try {
			SAXReader xmlReader = new SAXReader();
			xmlReader.setStripWhitespaceText(true);
			Document postprocessDoc = xmlReader.read(postprocessInputFile);
			postprocessorElement = postprocessDoc.getRootElement();
			if ( !postprocessorElement.getName().equalsIgnoreCase("OMDPostProcessor") )
				postprocessorElement = null;
			
		}  catch (DocumentException de) {
			System.out.println("Error attempting to parse: " + postprocessInputFile);
			postprocessorElement = null;
		}
	
	}
	
	public void begin() {
		openLogFile();		
		System.out.println("See logfile for output: " + logFile.toString());
		// if no PostProcessor tasks to do, then just leave
		if (postprocessorElement == null ) {
			logFileWriter.println("ERROR: Could not find a OMDPostProcessor element");
			logFileWriter.close();
			return;
		}
		
		// else get the list of PostProcessing tasks and do each one in turn

		logFileWriter.println("Begin post processing: " + postprocessInputFile);	
			
		tasks = postprocessorElement.elements("PostProcessTask");
		if ( tasks.size() == 0 ) {
			logFileWriter.println("ERROR: No PostProcessTask elements to process");
			logFileWriter.println("Completed Post processing: " + postprocessInputFile);
			logFileWriter.close();
			return;			
		}
		
		try {
			for (Iterator taskIterator = tasks.iterator();taskIterator.hasNext();) {
				Element task = (Element)taskIterator.next();
				PostProcessTask pptask = new PostProcessTask(task, logFileWriter);
				pptask.doTask();
			}
		}
		catch(Exception e) {
			logFileWriter.println("Encountered error on doing PostProcessTask");
			e.printStackTrace(logFileWriter);
			logFileWriter.close();
			System.exit(-1);
		}	
		
		logFileWriter.println("Completed Post processing: " + postprocessInputFile);
		logFileWriter.close();
	}
	
	private void openLogFile() {
		try {
			logFile = new File(postprocessInputFile + ".log");
			logFileWriter = new PrintWriter(new FileWriter(logFile));
		} catch (IOException ioe) {
			System.out.println("Error attempting to open: " + logFile);
			ioe.printStackTrace(System.out);
			System.exit(-1);
		}		
	}
	
	

	public static void main(String[] args) {
		String rootDirectory = args[0];
		String postprocessInputFile = args[1];
		OMDPostProcessor postprocessor = new OMDPostProcessor(rootDirectory, postprocessInputFile);
		postprocessor.openPostProcessInputFile();
		postprocessor.begin();		
	}
}

