/*
 * Created on May 7, 2004
 *
 */
package oldmcdata.postprocessors;

import java.io.PrintWriter;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *Copyright: 2000 - 2005
 *   Referentia Systems, Inc.
 * @author Stephen C. Upton
 *
 */
public class RowSelector {
	private String startsWithString, containsString;
	private PrintWriter logFileWriter;
	private String delimiter = ",";	
	private Pattern pattern = null;
	private PostProcessTask myTask;
	private Map myMapToUse;
	
	public RowSelector(PostProcessTask task, String mapToUse) {
		myTask = task;
		myMapToUse = myTask.getDataMap(mapToUse);
		this.configure();
	}
	
	public void configure() {
		logFileWriter = myTask.getLogFileWriter();
		this.setNumberString((String)myMapToUse.get(PostProcessTask.NUMBER));
		this.setStartsWithString((String)myMapToUse.get(PostProcessTask.STARTSWITH));
		this.setContainsString((String)myMapToUse.get(PostProcessTask.CONTAINS));
		this.setPattern((Pattern)myMapToUse.get(PostProcessTask.PATTERN));
	}
	
	public boolean lineMatches(String testline) {
		if ( startsWith(testline) )
			return true;
		if ( contains(testline) ) 
			return true;
		if ( matchesPattern(testline) )
			return true;	
			
		return false;
	}
	
	public boolean selectAllRows() {
		boolean result = false;
		if ( startsWithString == null && containsString == null &&
				pattern == null)
			result = true;
		return result;
	}

	
	public void setNumberString(String s) {
		logFileWriter.println("NOTICE: Row selector does not currently use NUMBER element");
	}
	
	public void setStartsWithString(String s) {
		startsWithString = s;		
	}
	
	public void setContainsString(String s) {
		containsString = s;
	}
	
	public void setPattern(Pattern p) {
		pattern = p;
	}
	
	
	private boolean startsWith(String testline) {
		if ( startsWithString == null )
			return false;
		String[] startsWithStrings = startsWithString.split(delimiter);
		for (int i=0;i<startsWithStrings.length;i++ ){
			boolean result = testline.startsWith(startsWithStrings[i]);
			if ( result ) 
				return true;  // as soon as found true, then return
		}
		return false;
	}
	
	private boolean contains(String testline) {
		if ( containsString == null )
			return false;
		String[] containStrings = containsString.split(delimiter);
		for (int i=0;i<containStrings.length;i++){
			int index = testline.indexOf(containStrings[i]);
			if ( index != -1 )
				return true;
		}
		
		return false;
	}
	
	private boolean matchesPattern(String testline) {
		if (pattern == null ) 
			return false;
		Matcher matcher = pattern.matcher(testline);
		if ( matcher.find() ) {
			matcher.reset();
			return true;
		}
		
		return false;
	}	
}
